package com.xynet.marketing.controller.merc;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xynet.marketing.enums.PromotionStatusEnum;
import com.xynet.marketing.enums.PromotionTypeEnum;
import com.xynet.marketing.service.PromotionService;
import com.xynet.marketing.utils.MercAuthUtils;
import com.xynet.marketing.utils.R;
import com.xynet.marketing.utils.collections.map.JHashMap;
import com.xynet.marketing.utils.collections.map.JMap;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 商家端促销活动控制器
 *
 * @author hechunping
 * @date 2025/2/12
 */
@RestController
@RequestMapping("merc/promotion")
@AllArgsConstructor
@Slf4j
@CrossOrigin(origins = "*")
public class PromotionController {

    private PromotionService promotionService;

    /**
     * 分页查询
     *
     * @param param
     * @param headers
     * @return
     */
    @PostMapping("page")
    private R page(@RequestBody Map<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        JMap<String, Object> jParam = new JHashMap<>(param);
        jParam.put("mercId", mercId);
        Page<Map<String, Object>> data = promotionService.page(jParam);
        return R.ok(data);
    }

    @PostMapping("obj")
    private R obj(@RequestBody Map<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        JMap<String, Object> jParam = new JHashMap<>(param);
        jParam.put("mercId", mercId);
        return R.ok(promotionService.obj(jParam));
    }

    @PostMapping("saveOrUpdate")
    private R saveOrUpdate(@RequestBody Map<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        JMap<String, Object> jParam = new JHashMap<>(param);
        jParam.put("mercId", mercId);
        return R.ok(promotionService.saveOrUpdate(jParam));
    }

    @PostMapping("updateEnableStatus")
    private R updateEnableStatus(@RequestBody Map<String, Object> param, @RequestHeader Map<String, String> headers) {
        Integer mercId = MercAuthUtils.getMercId(headers);
        JMap<String, Object> jParam = new JHashMap<>(param);
        jParam.put("mercId", mercId);
        promotionService.updateEnableStatus(jParam);
        return R.ok();
    }

    /**
     * 获取活动类型列表
     *
     * @return
     */
    @PostMapping("typeMaps")
    private R typeMaps() {
        return R.ok(PromotionTypeEnum.maps());
    }

    /**
     * 获取活动状态列表
     *
     * @return
     */
    @PostMapping("statusMaps")
    private R statusMaps() {
        return R.ok(PromotionStatusEnum.maps());
    }

}
