package com.xynet.marketing.utils.collections.map;

import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * map接口
 * function并不区分key的数据类型，假设泛型K为Integer，实际入参为String，而function同样可以获取到值
 */
public interface JMap<K, V> extends Map<K, V>, Serializable {

    /**
     * 覆盖写入
     */
    JMap<K, V> set(K key, V value);

    /**
     * 覆盖写入
     */
    JMap<K, V> set(boolean condition, K key, V value);




    /**
     * 不覆盖写入，key存在直接返回旧value，不存在则新增且返回null
     */
    V setIfAbsent(K key, V value);


    /**
     * 写入全部
     *
     * @param map
     * @return
     */
    JMap<K, V> setAll(Map<K, V> map);

    /**
     * 写入全部
     *
     * @param map
     * @return
     */
    JMap<K, V> setAll(boolean condition, Map<K, V> map);

    /**
     * 删除并返回value
     */
    V del(K key);



    String getString(K key);

    Integer getInt(K key);


    Long getLong(K key);

    Double getDouble(K key);

    BigDecimal getBigDecimal(K key);

    Boolean getBoolean(K key);

    LocalDateTime getLocalDateTime(K key);

    Timestamp getTimestamp(K key);

    V get(K key, Class<V> t);

    JMap<String, Object> getMap(K key);

    List<JMap<String, Object>> getList(K key);

}
